#!/usr/bin/env python
# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner


def run_tests(*test_args):
    if not test_args:
        test_args = ['tests']

    if 'DJANGO_SETTINGS_MODULE' not in os.environ:
        os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'

    django.setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(test_args)
    sys.exit(bool(failures))


if __name__ == '__main__':
    run_tests(*sys.argv[1:])



# coding=utf-8
# Based on https://github.com/wooyek/cookiecutter-django-website by Janusz Skonieczny

import logging
from pathlib import Path

import environ

logging.basicConfig(format='%(asctime)s %(levelname)-7s %(thread)-5d %(filename)s:%(lineno)s | %(funcName)s | %(message)s', datefmt='%H:%M:%S')
logging.getLogger().setLevel(logging.DEBUG)
logging.disable(logging.NOTSET)

logging.debug("Settings loading: %s" % __file__)

# This will read missing environment variables from a file
# We want to do this before loading any base settings as they may depend on environment
environ.Env.read_env(str(Path(__file__).parent / "local.env"), DEBUG='True')
from .base import *  # noqa: F402, F403 isort:skip

# https://docs.djangoproject.com/en/1.6/topics/email/#console-backend
# EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
# EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'
LOGGING['handlers']['mail_admins']['email_backend'] = 'django.core.mail.backends.dummy.EmailBackend'  # noqa: F405

DEFAULT_FILE_STORAGE = 'django.core.files.storage.FileSystemStorage'



# -*- coding: utf-8
from __future__ import absolute_import

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


def setup_app_settings():
    from . import app_settings as defaults
    from django.conf import settings
    for name in dir(defaults):
        if name.isupper() and not hasattr(settings, name):
            setattr(settings, name, getattr(defaults, name))


class DjangoOptOutConfig(AppConfig):
    name = 'django_opt_out'
    verbose_name = _('Messaging Opt-Outs')

    def ready(self):
        setup_app_settings()



# Generated by Django 2.0 on 2017-12-14 17:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_opt_out', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='optoutfeedbacktranslation',
            name='language',
            field=models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kab', 'Kabyle'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese')], max_length=5),
        ),
    ]



# coding=utf-8

import django.dispatch

list_unsubscribe = django.dispatch.Signal(providing_args=['request', 'email', 'data'])
link_unsubscribe = django.dispatch.Signal(providing_args=['request', 'email', 'data'])



# coding=utf-8

from django.conf import settings

# This will be used as a prefix when generating opt-out urls
# You cane leave it blank and set full path URL by yourself
OPT_OUT_BASE_URL = None

# If you set this to false we'll not validate that user actually used our opt-out url
# thus do not require confirmation of email address ownership.
# Anyone will be able to opt-pot anyone else.
OUT_OUT_REQUIRE_CONFIRMATION = True

# This is a secret for password generation
# We don use strong password hasher and email is the password base
# so you may want to use here something other than SECRET_KEY to not compromise main secret security
OPT_OUT_SECRET = settings.SECRET_KEY[::4]

# to validate that user actually got our message we put passwords int he opt-out url
# these have minimal security to we keep them simple and short
OPT_OUT_PASSWORD_HASHER = 'default'

# a custom view with goodbye message and suggestion to reconsider and change user feedback
# This view url must be similar to the OptOutSuccess view url
# It should not drop usage of secret as it ensures that a visitor is authenticated
# When providing your own please consider using OptOutBase as view base class
OPT_OUT_GOODBYE_VIEW = None

# a list of backends that should get enabled during application initialization
OPT_OUT_BACKENDS = []

# Allow Sparkpost plugin send_email to retry once on SMTPServerDisconnected, SMTPAuthenticationError
SPARKPOST_RETRY_ONCE = True

# Allow SparkPost plugin send_email to automatically render text message variant
SPARKPOST_HTML2TXT = True



# coding=utf-8
# Created 2014 by Janusz Skonieczny

"""
These heavily rely on staging settings and any modifications should consist only of keys and secrets.
Secrets should be loaded of the environment.

The goal here is to not change any settings may have impact on features present.
Changing keys and secrets should not hava that impact.
"""

import logging
import os
from pathlib import Path

import environ

logging.basicConfig(format='%(asctime)s %(levelname)-7s %(thread)-5d %(filename)s:%(lineno)s | %(funcName)s | %(message)s', datefmt='%H:%M:%S')
logging.getLogger().setLevel(logging.DEBUG)
logging.disable(logging.NOTSET)

logging.debug("Settings loading: %s" % __file__)

name = __name__.split('.')[-1].upper()
print("""
╔═════════════════════════════╗
║ LOADING PRODUCTION SETTINGS ║
╚═════════════════════════════╝
""")

# Set defaults for when env file is not present.
os.environ.update(DEBUG='False', ASSETS_DEBUG='False')

# This will read missing environment variables from a file
# We want to do this before loading any base settings as they may depend on environment
evironment_config = Path(__file__).with_suffix('.env')
if evironment_config.exists():
    environ.Env.read_env(str(evironment_config))

# noinspection PyUnresolvedReferences
from .base import *  # noqa: F402, F403 isort:skip

LOGGING['handlers']['console']['formatter'] = 'heroku'  # noqa: F405
LOGGING['handlers']['file'] = {  # noqa: F405
    'class': 'logging.handlers.RotatingFileHandler',
    'formatter': 'verbose',
    'backupCount': 3,
    'maxBytes': 4194304,  # 4MB
    'level': 'DEBUG',
    'filename': (os.path.join(BASE_DIR, 'logs', 'website.log')),  # noqa: F405
}
LOGGING['root']['handlers'].append('file')  # noqa: F405

log_file = Path(LOGGING['handlers']['file']['filename'])  # noqa: F405
if not log_file.parent.exists():  # pragma: no cover
    logging.info("Creating log directory: {}".format(log_file.parent))
    Path(log_file).parent.mkdir(parents=True)



# -*- coding: utf-8 -*-
import logging

from django.conf import settings
from django.db.models import Q
from django.shortcuts import redirect, resolve_url
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import TemplateView
from django.views.generic.edit import ModelFormMixin
from django_powerbank.views import Http403
from django_powerbank.views.auth import AbstractAccessView
from pascal_templates.views import CreateView, DetailView, UpdateView

from . import forms, models, signals
from .utils import validate_password

log = logging.getLogger(__name__)


class OptOutConfirm(CreateView):
    model = models.OptOut
    template_name = "django_opt_out/OptOut/form.html"
    form_class = forms.OptOutForm

    def email_confirmed(self):
        email = self.request.GET.get('email', None)
        auth = self.request.GET.get('auth', None)
        if email and auth:
            return validate_password(email, auth)

    def get_initial(self):
        return self.request.GET.dict()

    def get_form(self, form_class=None):
        form = super(OptOutConfirm, self).get_form(form_class)
        feedback = form.fields['feedback']
        tags = dict(self.get_tags())
        tags_filter = Q(tags__name__in=tags.keys())
        if tags.get('default', True) in (True, None, '', '1', 'True', 'true', 'on'):
            tags_filter |= Q(tags__isnull=True)
        feedback.queryset = feedback.queryset.filter(tags_filter)
        feedback.initial = list(feedback.queryset.filter(default=True).values_list('pk', flat=True))
        if self.email_confirmed():
            form.instance.confirmed = timezone.now()

        return form

    def get_context_data(self, **kwargs):
        kwargs['tags'] = self.get_tags()
        signals.send_signal(signals.opt_out_visited, self.__class__, view=self, request=self.request, context=kwargs)
        return super(OptOutConfirm, self).get_context_data(**kwargs)

    def form_valid(self, form):
        # noinspection PyAttributeOutsideInit
        self.object = form.save()
        self.save_tags()
        signals.send_signal(signals.opt_out_submitted, self.__class__, view=self, request=self.request, opt_out=self.object)
        return super(ModelFormMixin, self).form_valid(form)

    def get_tags(self):
        for name in self.request.GET.getlist('tag', []):
            yield name.split(":", 1) if ":" in name else (name, None)

    def save_tags(self):
        for name, value in self.get_tags():
            tag = models.OptOutTag.objects.filter(name=name).first()
            if tag is None:
                log.warning("Tag does not exist: %s", name)
                continue
            self.object.tags.create(tag=tag, value=value)

    def get_success_url(self):
        goodbye_view = settings.OPT_OUT_GOODBYE_VIEW or "django_opt_out:OptOutSuccess"
        return resolve_url(goodbye_view, pk=self.object.pk, secret=self.object.secret, email=self.object.email)


class OptOutBase(AbstractAccessView):
    def check_authorization(self, *args, **kwargs):
        # noinspection PyUnresolvedReferences
        item = self.get_object()
        email = self.kwargs['email']
        secret = self.kwargs['secret']
        if item.secret != secret or item.email != email:
            msg = _("Request authentication failed, secret or email is incorrect.")
            raise Http403(msg)


class OptOutSuccess(OptOutBase, DetailView):
    model = models.OptOut
    template_name = "django_opt_out/OptOut/success.html"

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.object.delete()
        signals.send_signal(signals.opt_out_deleted, self.__class__, view=self, request=self.request, opt_out=self.object)
        return redirect('django_opt_out:OptOutRemoved')


class OptOutRemoved(TemplateView):
    template_name = "django_opt_out/OptOut/removed.html"


class OptOutUpdate(OptOutBase, UpdateView):
    model = models.OptOut
    form_class = forms.OptOutForm
    template_name = "django_opt_out/OptOut/form.html"

    def get_form(self, form_class=None):
        form = super(OptOutUpdate, self).get_form(form_class)
        feedback = form.fields['feedback']
        tags = self.object.tags.all().values_list('tag__pk', flat=True)
        feedback.queryset = feedback.queryset.filter(Q(tags__in=tags) | Q(tags__isnull=True))
        feedback.initial = list(self.object.feedback.all().values_list('pk', flat=True))
        # form.fields['email'].readonly = True
        return form

    def get_success_url(self):
        return resolve_url("django_opt_out:OptOutSuccess", self.object.pk, self.object.secret, self.object.email)



# coding=utf-8
from email.utils import parseaddr

from django.conf import settings
from django.contrib.auth.hashers import check_password, make_password
from django.shortcuts import resolve_url
from six.moves.urllib.parse import urlencode


def get_opt_out_url(email, base_url=None, *tags):
    base_url = base_url or settings.OPT_OUT_BASE_URL or ''
    return base_url + get_opt_out_path(email=email, *tags)


def get_opt_out_path(email, *tags):
    password = get_password(email)
    name, email = parseaddr(email)
    query = [('email', email), ('tag', tags)]
    if settings.OUT_OUT_REQUIRE_CONFIRMATION:
        query.append(('auth', password))
    query = urlencode(query, doseq=True)
    url = resolve_url("django_opt_out:OptOutConfirm")
    return "?".join((url, query))


def get_password(email):
    password = settings.OPT_OUT_SECRET + email
    return make_password(password, hasher=settings.OPT_OUT_PASSWORD_HASHER)


def validate_password(email, encoded):
    password = settings.OPT_OUT_SECRET + email
    return check_password(password, encoded)



# coding=utf-8
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language

from . import models


class TranslatedMultipleChoiceField(forms.ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        return obj.trans(get_language())


class OptOutForm(forms.ModelForm):
    feedback = TranslatedMultipleChoiceField(
        label=_('Please help us provide a better service'),
        widget=forms.CheckboxSelectMultiple,
        queryset=models.OptOutFeedback.objects.all(),
        required=False)

    class Meta:
        model = models.OptOut
        fields = ('email', 'feedback', 'comment')
        labels = {
            'comment': _('Please tell us what can we do better'),
        }

    def save(self, commit=True):
        item = super().save(commit)
        if commit:
            feedback = models.OptOutFeedback.objects.filter(pk__in=self.cleaned_data['feedback'])
            item.feedback.add(*feedback)
        return item



# -*- coding: utf-8
# Based on https://github.com/wooyek/cookiecutter-django-website by Janusz Skonieczny

import logging
import os
from pathlib import Path

import environ

logging.basicConfig(format='%(asctime)s %(levelname)-7s %(thread)-5d %(filename)s:%(lineno)s | %(funcName)s | %(message)s', datefmt='%H:%M:%S')
logging.getLogger().setLevel(logging.DEBUG)
logging.disable(logging.NOTSET)

logging.debug("Settings loading: %s" % __file__)

print("""
╭─────────{border}─╮
│ Loading {name} │
╰─────────{border}─╯
""".format(name=__name__, border='─' * len(__name__)))

# This will read missing environment variables from a file
# We want to do this before loading any base settings as they may depend on environment
# NOTE: Django test runner override DEBUG to False, use override_settings decorator to change this
environ.Env.read_env(str(Path(__file__).with_suffix('.env')), DEBUG='False')

# We rely here on example_project base settings
from website.settings.base import *  # noqa: F402, F403 isort:skip

if "DATABASE_URL" not in os.environ:  # pragma: no cover
    DATABASES['default']['NAME'] = ':memory:'  # noqa F405

ROOT_URLCONF = 'tests.urls'

# https://docs.djangoproject.com/en/1.6/topics/email/#console-backend
# EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'
LOGGING['handlers']['mail_admins']['email_backend'] = 'django.core.mail.backends.dummy.EmailBackend'  # noqa: F405

DEFAULT_FILE_STORAGE = 'django.core.files.storage.FileSystemStorage'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `django-opt-out` package."""
import unittest

import django
import pytest
from click.testing import CliRunner
from django.test import TestCase

import django_opt_out
from django_opt_out import cli, factories, models

django_opt_out.__version__


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/wooyek/cookiecutter-pylib')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert 'django_opt_out.cli.main' in result.output
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output


class SetupDefaultsTests(TestCase):
    @unittest.skipIf(django.VERSION[0] > 1, "django-import-export does not support django 2")
    def test_tags(self):
        from django_opt_out.management.commands.opt_out_feedback_defaults import Command
        Command().import_all()
        self.assertEqual(3, models.OptOutTag.objects.count())
        self.assertEqual(8, models.OptOutFeedback.objects.count())
        self.assertEqual(8, models.OptOutFeedbackTranslation.objects.count())

    def test_ignore_call_on_data(self):
        factories.OptOutTagFactory()
        from django_opt_out.management.commands.opt_out_feedback_defaults import Command
        Command().handle(force=False, on_empty=True)

    def test_failt_on_existing_data(self):
        factories.OptOutTagFactory()
        from django_opt_out.management.commands.opt_out_feedback_defaults import Command
        self.assertRaises(AssertionError, Command().handle, force=False, on_empty=False)



# coding=utf-8
import logging

import sparkpost
from django.conf import settings
from django.dispatch import receiver
from django.utils.functional import SimpleLazyObject
from sparkpost.exceptions import SparkPostAPIException

from . import signals
from ... import signals as opt_out

log = logging.getLogger(__name__)


@receiver(signals.list_unsubscribe, dispatch_uid="create_opt_out")
@receiver(signals.link_unsubscribe, dispatch_uid="create_opt_out")
def create_opt_out(sender, request, email, data, **kwargs):
    log.debug("OptOut %s on SparkPost unsubscribe event", email)
    from django_opt_out.models import OptOut
    OptOut.objects.create(email=email, data=data)


@receiver(opt_out.opt_out_submitted, dispatch_uid="suppress_email")
def suppress_email(sender, view, request, opt_out, **kwargs):
    log.debug("Creating suppression for %s in SparkPost", opt_out.email)
    client.suppression_list.create({
        'email': opt_out.email,
        'transactional': True,
        'non-transactional': True,
        'description': 'Created through: {}'.format(opt_out)}
    )


@receiver(opt_out.opt_out_deleted, dispatch_uid="remove_suppression")
def remove_suppression(sender, view, request, opt_out, **kwargs):
    log.debug("Removing suppression %s from SparkPost", opt_out.email)
    try:
        client.suppression_list.delete(opt_out.email)
    except SparkPostAPIException as ex:
        if ex.status != 404:
            raise ex


def get_client(setting='SPARKPOST_API_KEY'):
    return sparkpost.SparkPost(getattr(settings, setting, None))


client = SimpleLazyObject(get_client)



# coding=utf-8
import json
import logging
from smtplib import SMTPAuthenticationError, SMTPServerDisconnected

from django import dispatch
from django.conf import settings
from django.core import mail
from django.template import TemplateDoesNotExist
from django.template.loader import render_to_string
from sparkpost.exceptions import SparkPostAPIException

log = logging.getLogger(__name__)
send_mail_template_signal = dispatch.Signal(providing_args=["message", "template", "ctx"])


def send_email(subject, to, ctx, template_html, template_txt=None, **kwargs):
    """Send rendered message with SparkPost unsubscribe support"""

    html_message, text_message = _render_message(ctx, template_html, template_txt)
    message = _message_factory(html_message, text_message, subject=subject, to=to, **kwargs)
    log.debug("%s: %s", to, subject)

    if 'unsubscribe' in ctx:
        message.extra_headers['List-Unsubscribe'] = "<{}>".format(ctx['unsubscribe'])
        message.extra_headers['X-MSYS-API'] = json.dumps({'options': {'transactional': True}})

    try:
        return _send_robust(message)
    finally:
        send_mail_template_signal.send_robust(sender=send_email, message=message, template_html=template_html, ctx=ctx)


def _message_factory(html_message, text_message, **kwargs):
    to = kwargs.pop('to')
    if not isinstance(to, list):
        to = [to]
    kwargs.setdefault('from_email', settings.DEFAULT_FROM_EMAIL)
    try:
        kwargs.setdefault('reply_to', [settings.DEFAULT_REPLY_TO_EMAIL])
    except AttributeError:
        pass
    message = mail.EmailMultiAlternatives(body=text_message, to=to, **kwargs)
    if html_message:
        message.attach_alternative(html_message, 'text/html')
    return message


def _send_robust(message):
    try:
        return message.send()
    except SparkPostAPIException as ex:
        if ex.status == 1902 or not settings.FAIL_ON_EMAIL_SUPPRESSION:
            logging.error("Email suppression: %s", message.to, exc_info=ex)
        else:
            raise
    except (SMTPServerDisconnected, SMTPAuthenticationError) as ex:
        if not settings.SPARKPOST_RETRY_ONCE:
            raise
        logging.error("", exc_info=ex)
        # Retry once
        return message.send()


def _render_message(ctx, template_html, template_txt):
    html_message = render_to_string(template_html, ctx)
    text_message = None
    if template_txt is None:
        template_txt = (template_html.replace(".html", ".jinja2"), template_html.replace(".html", ".txt"))
    try:
        text_message = render_to_string(template_txt, ctx)
    except TemplateDoesNotExist:
        if settings.SPARKPOST_HTML2TXT:
            import html2text
            text_message = html2text.html2text(html_message)
    return html_message, text_message



# coding=utf-8
import json
import logging

from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from . import signals
from ...signals import send_signal

log = logging.getLogger(__name__)


class SparkPostUnsubscribeWebhook(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, *args, **kwargs):
        return super(SparkPostUnsubscribeWebhook, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        data = json.loads(request.body.decode("UTF-8"))
        self.process_data(data)
        return HttpResponse("Thx!")

    def process_data(self, data):
        log.debug("data: %s", data)
        for entry in data:
            event = entry.get('msys', {}).get('unsubscribe_event', {})
            event_type = event.get('type')
            if event_type == 'list_unsubscribe':
                email = event['raw_rcpt_to']
                send_signal(signals.list_unsubscribe, self.__class__, request=self.request, email=email, data=entry)
            elif event_type == 'link_unsubscribe':
                email = event['raw_rcpt_to']
                send_signal(signals.link_unsubscribe, self.__class__, request=self.request, email=email, data=entry)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

import django
import pytest
from django.test import SimpleTestCase, TestCase
from django.test.utils import override_settings
from django_powerbank.testing.base import MigrationsCheckMx
from mock import patch
from six.moves.urllib.parse import quote_plus

from django_opt_out import factories
from django_opt_out.utils import get_opt_out_path, get_opt_out_url, get_password, validate_password


@pytest.mark.django_db
class MigrationsCheckTests(MigrationsCheckMx, TestCase):

    @unittest.skipIf(django.VERSION[0] < 2, "There maybe difference in migrations state for older versions of django")
    def test_missing_migrations(self):
        super().test_missing_migrations()


@override_settings(OUT_OUT_REQUIRE_CONFIRMATION=False)
class TestOptOutUrl(SimpleTestCase):
    def test_no_tags(self):
        url = get_opt_out_path('foo@bar.com')
        self.assertEqual(url, '/opt-out/?email=foo%40bar.com')

    def test_one_tag(self):
        url = get_opt_out_path('foo2@bar.com', 'notification')
        self.assertEqual(url, '/opt-out/?email=foo2%40bar.com&tag=notification')

    def test_multiple_tags(self):
        url = get_opt_out_path('foo@bar.com', 'notifications', 'comments')
        self.assertEqual(url, '/opt-out/?email=foo%40bar.com&tag=notifications&tag=comments')

    def test_many_tags(self):
        url = get_opt_out_path('foo@bar.com', 'notifications', 'comments')
        self.assertEqual(url, '/opt-out/?email=foo%40bar.com&tag=notifications&tag=comments')

    def test_tags_values(self):
        url = get_opt_out_path('foo@bar.com', 'notifications:123', 'comments:owner')
        self.assertEqual(url, '/opt-out/?email=foo%40bar.com&tag=notifications%3A123&tag=comments%3Aowner')

    def test_named_email_parsing(self):
        url = get_opt_out_path('Artur Dent <42@galaxy.net>')
        self.assertEqual(url, '/opt-out/?email=42%40galaxy.net')

    @override_settings(OPT_OUT_BASE_URL='https://ex.com')
    def test_settings_base_url(self):
        url = get_opt_out_url('foo@bar.com')
        self.assertEqual(url, 'https://ex.com/opt-out/?email=foo%40bar.com')

    @override_settings(OUT_OUT_REQUIRE_CONFIRMATION=True)
    @patch('django.contrib.auth.hashers.BasePasswordHasher.salt')
    def test_password_in_url(self, salt):
        salt.return_value = 'salty'
        opt_out = get_opt_out_url('foo@bar.com', base_url='http://by.com')
        url = 'http://by.com/opt-out/?email=foo%40bar.com'
        url += "&auth=" + quote_plus(get_password('foo@bar.com'))
        self.assertEqual(2, salt.call_count)
        self.assertEqual(url, opt_out)


class PasswordTests(SimpleTestCase):
    @patch('django.contrib.auth.hashers.BasePasswordHasher.salt')
    def test_valid(self, salt):
        salt.return_value = 'salty'
        email = "foo@bar.com"
        encoded = get_password(email)
        self.assertTrue(validate_password(email, encoded))


class OptOutFeedbackTest(TestCase):
    def test_trans_fallback(self):
        item = factories.OptOutFeedbackFactory()
        self.assertEqual(item.text, item.trans())

    def test_trans(self):
        text = 'Zażółć gęślą jaźń'
        item = factories.OptOutFeedbackTranslationFactory(text=text).feedback
        trans = item.trans('pl')
        self.assertEqual(text, trans)
        self.assertNotEqual(item.text, trans)
